"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.editString = void 0;
var editString = function (input, changes) {
    var output = input;
    for (var i = 0, n = changes.length; i < n; i++) {
        var _a = changes[i], start = _a.start, end = _a.end, value = _a.value;
        var offsetStartIndex = start;
        var offsetEndIndex = end;
        var invalid = false;
        var insertion = start === end;
        // based on all of the previous edits, calculate where this edit is
        for (var j = 0; j < i; j++) {
            var _b = changes[j], previousStartIndex = _b.start, previousEndIndex = _b.end, previousNewValue = _b.value;
            var prevInsertion = previousStartIndex === previousEndIndex;
            var startIndicesMatch = start === previousStartIndex;
            var endIndicesMatch = end === previousEndIndex;
            // input :  a b c d e f g h i
            // prev  :     ^-------^
            // ✔     :     ^
            var insertBeginning = startIndicesMatch && insertion;
            // input :  a b c d e f g h i
            // prev  :     ^-------^
            // ✔     :             ^
            var insertEnd = endIndicesMatch && insertion;
            // input :  a b c d e f g h i
            // prev  :     ^
            // ✔     :     ^-------^
            var prevInsertBeginning = startIndicesMatch && prevInsertion;
            // input :  a b c d e f g h i
            // prev  :     ^
            // ✔     :     ^-------^
            var prevInsertEnd = endIndicesMatch && prevInsertion;
            var currOrPrevInserting = insertBeginning || insertEnd || prevInsertBeginning || prevInsertEnd;
            // input :  a b c d e f g h i
            // prev  :         ^-------^
            // ✔     :     ^-------^
            if (previousStartIndex < end && previousStartIndex > start) {
                offsetEndIndex = offsetEndIndex - (end - previousStartIndex);
            }
            // input :  a b c d e f g h i
            // prev  :   ^-----^
            // ✔     :       ^-------^
            if (previousEndIndex > start && previousEndIndex < end) {
                offsetStartIndex = offsetStartIndex + (previousEndIndex - start);
            }
            // Invalid edit because previous replacement
            // completely clobbers this one. There's nothing else to edit.
            // input :  a b c d e f g h i
            // prev  :   ^---------^
            // ✔     :     ^---^
            // ✔     : ^-------------^
            // ✘     :   ^
            // ✘     :             ^
            // ✘     :   ^-----------^
            if (((start >= previousStartIndex && end <= previousEndIndex) ||
                (start < previousStartIndex && end >= previousEndIndex)) &&
                !currOrPrevInserting) {
                invalid = true;
                break;
            }
            // input :  a b c d e f g h
            // prev  :     ^-----^
            // ✔     :       ^-----^
            // ✔     :           ^---^
            // ✔     :               ^-^
            // ✔     : ^-----^
            // ✘     : ^---^
            // ✘     :   ^-^
            // ✘     :     ^
            // input :  a b c d e f g h
            // prev  : ^---^
            // ✔     :   ^---^
            if (previousStartIndex <= start && end > previousStartIndex) {
                var prevValueLengthDelta = previousNewValue.length - (previousEndIndex - previousStartIndex);
                // shift left or right
                offsetStartIndex = Math.max(0, offsetStartIndex + prevValueLengthDelta);
                offsetEndIndex = Math.max(0, offsetEndIndex + prevValueLengthDelta);
            }
        }
        if (!invalid) {
            output =
                output.substr(0, offsetStartIndex) +
                    value +
                    output.substr(offsetEndIndex);
        }
    }
    return output;
};
exports.editString = editString;
